// iodng header file
// defines the messages exchanged between nfsd and iod
//
// WARNING:
// Some compiler properties are implied:
// If struct { int x; char o[0]; int y; } s then offsetof(o) == offsetof(y)
//
// WARNING (ugly hack)
// THIS STRUCTURE IS DIRECTLY SENT ON THE WIRE AND THIS IS EVIL !!!
// It depends on compiler padding etc so use the same compiler to compile the
// sources for the nfspd and for the iodng.
//
// Author: Pierre Lombard <pierre.lombard@imag.fr>
// Licence: GPL

#ifndef __IODNG_H__
#define __IODNG_H__ 1

#include "default.h"

#include <rpc/rpc_msg.h>
#include "nfs_prot.h"


#define NFSP_MAGIC_MSG		0xBAFE
#define NFSP_READ_REQ		0x0001
#define NFSP_WRITE_REQ		0x0002
#define NFSP_WRITEMIRROR_REQ	0x0003
#define NFSP_FWDREAD_1_REQ	0x0010
#define NFSP_FWDREAD_2_REQ	0x0011
#define NFSP_FWDWRITE_1_REQ	0x0020
#define NFSP_FWDWRITE_2_REQ	0x0021
#define NFSP_PURGE_REQ		0x0030
#define NFSP_PING_REQ		0x0040
#define NFSP_PONG_REQ		0x0041
#define NFSP_MAX_REQ		0x0050



// The format of the messages used to communicate with iods.
// These messages must be delivered by means of UDP packets.
// WARNING: if the layout of this structure is changed
//          DO check the #define's below used to compute packet size
typedef struct {
	uint16_t magic;				// must be MAGIC_MSG_T
	uint16_t msgtype;			// one of the NFSP_*_REQ

	char _MINIHEADER[0];

	uint32_t client_addr;			// client IP address
	uint32_t server_addr;			// server IP address
	uint16_t client_port;			// client port number
	uint16_t server_port;			// server port number

	unsigned char client_mac_addr[6];	// client MAC address

	char _STARTS[0];

	union {
		struct {
			u_long xid;			// same type as rpc_msg.rm_xid in rpc/rpc_msg.h
			u_int inode;			// the real inode of the metafile
			unsigned int seed;		// the seed of the file
			u_int stripe_bs;		// size in bytes of a storage block
			u_int stripe_w;			// stripe width (number of (v)iods used for storage)
			fattr attributes;		// file attributes as defined in nfs_prot.{x,h}

			u_int offset;			// offset of the requested operation

			u_int datalen;			// length of data to read or write
			char _ENDS[0];
			char dataval[NFS_MAXDATA];	// nothing (size=0) if READ or data if WRITE
		} _RDWR;
#ifdef WRITEMIRROR_SUPPORT
		struct {
			u_long xid;			// same type as rpc_msg.rm_xid in rpc/rpc_msg.h

			u_int inode;			// the real inode of the metafile
			unsigned int seed;		// the seed of the file
			u_int stripe_bs;		// size in bytes of a storage block
			u_int stripe_w;			// stripe width (number of (v)iods used for storage)
			fattr attributes;		// file attributes as defined in nfs_prot.{x,h}
			u_int offset;			// offset of the requested operation

			// If idx == -1 then the iod must return the NFS ack to the client
			// If idx >=0 then the iod forwards the data to the
			//   next iod in the list that is nextiods[idx] (and
			//   decrease the idx in the message)
			//
			// Example:
			//   S: Serveur
			//   I1, I2, I3: iods
			//   In the message, nextiods[0]=I2 nextiods[1]=I3
			//
			//   S to I1:  idx=1   send to nextiods[idx] with idx set to 1-1
			//   I1 to I3: idx=0   send to nextiods[idx] with idx set to 0-1
			//   I3 to I2: idx=-1  send NFS ack to client
			int idx;					// still to mirror in 0..MAX_NEXTIODS-1
			struct sockaddr_in nextiods[MAX_NEXTIODS];	// nextiods[idx .. 0] are used
			
			u_int datalen;
			char _ENDS[0];
			char dataval[NFS_MAXDATA];	// nothing (size=0) if READ or data if WRITE
		} _WMR;
#endif /* WRITEMIRROR_SUPPORT */

		struct {
			u_long xid;			// same type as rpc_msg.rm_xid in rpc/rpc_msg.h
			u_int inode;			// the real inode of the metafile
			unsigned int seed;		// the seed of the file
			u_int stripe_bs;		// size in bytes of a storage block
			u_int stripe_w;			// stripe width (number of (v)iods used for storage)
			fattr attributes;		// file attributes as defined in nfs_prot.{x,h}

			u_int offset;			// offset of the requested operation

			struct sockaddr_in nextiod;	// next hop (iod1 then null)
			u_int nextlen;			// next len to read (0 then what's still to read)

			u_int datalen;			// total length of data to read (NOT CHANGED)
			char _ENDS[0];
			char dataval[NFS_MAXDATA];	// void then DATA already read
		} _FWDRDWR;

		struct {
			int nfiles;
			struct {
				u_int inode;		// the real inode of the metafile
				unsigned int seed;	// the seed of file
			} files[MAXPENDINGFILES];
			char _ENDS[0];
		} _PURGE;

		struct {
			u_long seq;			// The sequence number
			char _ENDS[0];
		} _PING;

	} _KIND;
} msg_iod_t;

//#define rdwrreq _KIND._RDWR
#define readreq _KIND._RDWR
#define writereq _KIND._RDWR
#ifdef WRITEMIRROR_SUPPORT
#define writemirrorreq _KIND._WMR
#endif /* WRITEMIRROR_SUPPORT */
#define fwdrdwrreq _KIND._FWDRDWR
#define fwdreadreq _KIND._FWDRDWR
#define fwdwritereq _KIND._FWDRDWR
#define purgereq _KIND._PURGE
#define pingreq _KIND._PING

#define OFFSETOF(TYPE, MEMBER)		((size_t) &((TYPE *)0)->MEMBER)
#define MSG_IOD_READ_LEN		(OFFSETOF(msg_iod_t,_KIND._RDWR._ENDS))
#define MSG_IOD_WRITE_LEN(nbytes)	(OFFSETOF(msg_iod_t,_KIND._RDWR._ENDS) + (nbytes))
#ifdef WRITEMIRROR_SUPPORT
#define MSG_IOD_WRITE_MIRROR_LEN(nbytes)	(OFFSETOF(msg_iod_t,_KIND._WMR._ENDS) + (nbytes))
#endif /* WRITEMIRROR_SUPPORT */
#define MSG_IOD_FWDREAD_LEN(nbytes)	(OFFSETOF(msg_iod_t,_KIND._FWDRDWR._ENDS) + (nbytes))
#define MSG_IOD_FWDWRITE_LEN(nbytes)	(OFFSETOF(msg_iod_t,_KIND._FWDRDWR._ENDS) + (nbytes))
#define MSG_IOD_PURGE_LEN		(OFFSETOF(msg_iod_t,_KIND._PURGE._ENDS))
#define MSG_IOD_PING_LEN		(OFFSETOF(msg_iod_t,_KIND._PING._ENDS))

#define MSG_IOD_MINIHEADER_LEN		(OFFSETOF(msg_iod_t,_MINIHEADER))
#define MSG_IOD_COMMON_LEN		(OFFSETOF(msg_iod_t,_STARTS))
#define MSG_IOD_LEN_MAX			(sizeof(msg_iod_t))

#endif

