// This is used to do the mapping (control messages from user) versus
// (procedures) and gathers the I/O on the UNIX socket and the dispatch
//  
//
// Author: Pierre Lombard <pierre.lombard@imag.fr> 2002/07 GPLv2

#ifdef VIODS_SUPPORT

//#include "dbg.h"

#include "sure_sock.h"

#include "libnfspdctl/nfspd_stub.h"
#include "nfsp_viods.h"

#include <sys/types.h>
#include <sys/socket.h>


// TODO: think of the case when there are errors in sure_send() => should the
// modification be effective ?  (I think it should be ok since we're with unix
// sockets -- pl)

int nfsp_iodctl (int fd) {
	int result, ret;
	struct message_t mess;

	ret = sure_recv(fd, &mess, sizeof(mess), MSG_NOSIGNAL);
	if (ret != sizeof(mess)) {
		ERR("sure_recv failed errno=%d", errno);
		return -1;
	}

	if (mess.magic != NFSP_MAGIC) {
		ERR("not enough magic");
		return -1;
	}

	switch (mess.fnct) {
	case NFSP_ADD_IOD:
		result = nfspd_addiod_svc(&mess.addr);
		break;
	case NFSP_FREE_IOD:
		result = nfspd_freeiod_svc(mess.ih);
		break;
	case NFSP_BIND:
		result = nfspd_bindiod_svc(mess.iviod, mess.ih);
		break;
	case NFSP_UNBIND:
		result = nfspd_unbindiod_svc(mess.ih);
		break;
	case NFSP_SETNBVIODS:
		result = nfspd_setnbviods_svc(mess.nbviods);
		break;
	case NFSP_GETNBVIODS:
		result = nfspd_getnbviods_svc();
		break;
	case NFSP_DROP:
		result = nfspd_drop_svc();
		break;
	case NFSP_FORWARD:
		result = nfspd_forward_svc();
		break;
	case NFSP_FLUSH:
		result = nfspd_flush_svc();
		break;
	case NFSP_GET_VIOD_SIZE:
		result = nfspd_get_viod_size_svc(mess.iviod);
		break;
	default:
		ERR("unknown message type %d", mess.fnct);
		result = -1;
	}

	ret = sure_send (fd, &result, sizeof (result), MSG_NOSIGNAL);
	if (ret != sizeof (result)) {
		return -1;
	}
	return 0;
}

#endif /* VIODS_SUPPORT */

